/* Copyright (c) 2017, EPFL/Blue Brain Project
 *                     Mohamed-Ghaith Kaabi <mohamed.kaabi@epfl.ch>
 *
 * This file is part of Brion <https://github.com/BlueBrain/Brion>
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3.0 as published
 * by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef BRION_PLUGIN_SPIKEREPORTASCII_H
#define BRION_PLUGIN_SPIKEREPORTASCII_H

#include "../pluginInitData.h"

#include <brion/spikeReportPlugin.h>
#include <brion/types.h>

namespace brion
{
namespace plugin
{
class SpikeReportASCII : public SpikeReportPlugin
{
public:
    explicit SpikeReportASCII(const PluginInitData& initData);

    Spikes read(float min) final;
    Spikes readUntil(float toTimeStamp) final;
    void readSeek(float toTimeStamp) final;
    void writeSeek(float toTimeStamp) final;
    bool supportsBackwardSeek() const final { return true; }
protected:
    Spikes _spikes;
    Spikes::iterator _lastReadPosition;

    // Returns true if parsing succeeded
    using ParseFunc = std::function<bool(const std::string&, Spike&)>;

    using WriteFunc = std::function<void(std::ostream&, const Spike&)>;

    static Spikes parse(const Strings& files, const ParseFunc& parse);
    static Spikes parse(const std::string& filename, const ParseFunc& parse);
    void append(const Spike* spikes, size_t size, const WriteFunc& writeFunc);
};
}
}

#endif // BRION_PLUGIN_SPIKEREPORTFILEBASED_H
